package DX::ShellSession;

use Tcl;
use Scalar::Util qw(weaken);
use DX::Class;

has shell_state => (is => 'rwp', required => 1, isa => ShellState);

has tcl => (is => 'lazy', builder => sub {
  my ($self) = @_;
  weaken $self;
  my $tcl = Tcl->new;
  $tcl->CreateCommand('?' => sub {
    $self->apply_to_state([ 'new_query_state' ], [ mode => 'query' ])
  });
  $tcl->CreateCommand('?+' => sub {
    $self->apply_to_state([ mode => 'query' ])
  });
  $tcl->CreateCommand('.' => sub {
    $self->apply_to_state([ mode => 'shell' ])
  });
  return $tcl;
});

sub apply_to_state {
  my ($self, @to_apply) = @_;
  my $state = $self->shell_state;
  our @Result;
  foreach my $to_apply (@to_apply) {
    my ($change, @args) = @$to_apply;
    ($state, my @this_result) = $state->${\"with_${change}"}(@args);
    push @Result, @this_result;
  }
  $self->_set_shell_state($state);
  return;
}

sub is_complete_command_string {
  my ($self, $string) = @_;
  return !!$self->tcl->icall(info => complete => $string);
}

sub eval_command_string {
  my ($self, $string) = @_;
  local our @Result;
  $self->tcl->Eval($string);
  return @Result;
}

1;
